/**
 * \file dxProtocol.h
 * \brief Defines with the instruction codes for the detiDx gateway

General communication protocol with board Dynamixel-UA:\n
general packet: <b>STX CC SS NN XX YY ZZ .. ED CS</b>\n
Where:\n
CC - Command\n
SS - Device Series (0: AX, 1: RX, 2: MX)\n
NN - Number of bytes of data field\n
CS - Checksum\n
STX - Start of packet '%'\n
ED - End of Data '&'\n
\n
For the examples of each command below:\n
AA - Address\n
VV - Value\n
VH - Value High\n
VL - Value Low\n
PH - Position High\n
PL - Position Low\n
SH - Speed High\n
SL - Speed Low\n
BB - Byte returned from "Read byte" command\n
WH - High byte returned from "Read Word" command\n
WL - Low byte returned from "Read Word" command\n
\n
ER - Error byte (generated by device). If bit7 is set, Timeout error (device did not respond)\n
 */
#ifndef _DXPROTO_H_
#define _DXPROTO_H_

/*!
Command: STX 00 SS 01 ID ED CS\n
Answer:  STX 00 SS 01 ER ED CS
*/
#define DX_RESET			0x00

/*!
Command:  STX 01 SS 01 ID ED CS\n
Answer:   STX 01 SS 01 ER ED CS
*/
#define DX_PING				0x01

/*!
Command:  STX 02 SS 00 ED CS\n
Answer:   STX 02 SS 00 ED CS\n
<b>(Devices do not answer to this command)</b>
*/
#define DX_ACTION			0x02

/*!
Command:	STX 03 SS 02 ID AA ED CS\n
Answer:	STX 03 SS 02 ER BB ED CS
*/
#define DX_READ_BYTE		0x03

/*!
Command:	STX 04 SS 03 ID AA VV ED CS\n
Answer:  STX 04 SS 01 ER ED CS\n
	<b>(if the ID of the destination device is "Broadcast", no answer will be provided by any device. In such case the
	answer is: STX 04 SS 00 ED CS)</b>
*/
#define DX_WRITE_BYTE		0x04

/*!
Command:	STX 05 SS 02 ID AA ED CS\n
Answer:	STX 05 SS 03 ER WL WH ED CS
*/
#define DX_READ_WORD		0x05

/*!
Command:	STX 06 SS 04 ID AA VL VH ED CS\n
Answer:  STX 06 SS 01 ER ED CS\n
	<b>(if the ID of the destination device is "Broadcast", no answer will be provided by any device. In such case the
	answer is: STX 06 SS 00 ED CS)</b>
*/
#define DX_WRITE_WORD		0x06

/*!
Command:	STX 07 SS NN AA ID1 VV1 ID2 VV2 ... ED CS\n
Answer:	STX 07 SS 00 ED CS
*/
#define DX_SYNC_WRITE_BYTE	0x07

/*!
Command:	STX 08 SS NN AA ID1 VL1 VH1 ID2 VL2 VH2 ... ED CS\n
Answer:	STX 08 SS 00 ED CS
*/
#define DX_SYNC_WRITE_WORD	0x08

/*!
Command:	STX 09 SS 03 ID AA VV ED CS\n
Answer:  STX 09 SS 01 ER ED CS\n
<b>(if the ID of the destination device is "Broadcast", no answer will be provided by any device. In such case the
	answer is: STX 09 SS 00 ED CS)</b>
*/
#define DX_REG_WRITE_BYTE	0x09

/*!
Command:	STX 0A SS 04 ID AA VL VH ED CS\n
Answer:  STX 0A SS 01 ER ED CS\n
	<b>(if the ID of the destination device is "Broadcast", no answer will be provided by any device. In such case the
	answer is: STX 0A SS 00 ED CS)</b>
*/
#define DX_REG_WRITE_WORD	0x0A

/*!
Command:	STX 10 SS 05 ID PL PH SL SH ED CS\n
Answer:  STX 10 SS 01 ER ED CS\n
	<b>(if the ID of the destination device is "Broadcast", no answer will be provided by any device. In such case the
	answer is: STX 10 SS 00 ED CS)</b>
*/
#define DX_GOTO_POSITION	0x10

/*!
Command:	STX 11 SS 05 ID PL PH SL SH ED CS\n
Answer:  STX 11 SS 01 ER ED CS\n
	<b>(if the ID of the destination device is "Broadcast", no answer will be provided by any device. In such case the
	answer is: STX 11 SS 00 ED CS)</b>
*/
#define DX_WRITE_POSITION	0x11

/*!
Command:	STX 12 SS 01 ID ED CS\n
Answer:	STX 12 SS 03 ER PL PH ED CS
*/
#define DX_READ_POSITION	0x12

#endif
